//+------------------------------------------------------------------+
//|                                             SignalNewsFilter.mqh |
//|                                                     AnimateDread |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "AnimateDread"
#property link      "https://www.mql5.com"
#include "..\Expert\ExpertSignalCustom.mqh"

// Define NEWS_IMPACT_HIGH with your desired impact level
#define NEWS_IMPACT 1 // Replace with the desired impact level (e.g., 3 for high impact)
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalNewsFilter : public CExpertSignalCustom
  {
protected:
   // Input parameters
   long              m_minImpact;            // Minimum news impact to trigger signal
   int               m_lookbackMinutes;       // Lookback period in minutes

public:
                     CSignalNewsFilter();
                    ~CSignalNewsFilter();

   // Method to set input parameters
   void              SetMinImpact(long minImpact) { m_minImpact = minImpact; }
   void              SetLookbackMinutes(int lookbackMinutes) { m_lookbackMinutes = lookbackMinutes; }

   // Method to check news filter conditions
   virtual double    Direction();
  };

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSignalNewsFilter::CSignalNewsFilter() : m_minImpact(NEWS_IMPACT), m_lookbackMinutes(30)
  {
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSignalNewsFilter::~CSignalNewsFilter()
  {
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSignalNewsFilter::Direction()
  {
   if(m_lookbackMinutes == -1)
      return 0.0;
// Get the current date and time
   datetime currentDate = TimeCurrent();
// Calculate the lookback time based on the current date and time
   datetime lookbackTime = currentDate - m_lookbackMinutes * 60;
// Create an array to store economic calendar values
   MqlCalendarValue values[];
// Set the boundaries of the interval to retrieve news events
   datetime date_from = lookbackTime;  // Start from the calculated lookback time
   datetime date_to = currentDate;     // End at the current date and time
// Request news events history within the specified time interval
   if(CalendarValueHistory(values, date_from, date_to))
     {
      // Loop through the retrieved news events
      for(int i = 0; i < ArraySize(values); i++)
        {
         // Check if the event's impact_type is greater than or equal to the minImpact parameter
         if(values[i].impact_type >= m_minImpact && values[i].time >= lookbackTime)
           {
            // High-impact news event found within the lookback period
            return EMPTY_VALUE;
           }
        }
     }
// No high-impact news events within the lookback period
   return 0.0;
  }
//+------------------------------------------------------------------+
