//+------------------------------------------------------------------+
//|                                                      ProjectName |
//|                                      Copyright 2020, CompanyName |
//|                                       http://www.companyname.net |
//+------------------------------------------------------------------+
#include "..\Expert\ExpertSignalCustom.mqh"
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of session filter                                  |
//| Type=SignalAdvanced                                              |
//| Name=SessionFilter                                               |
//| ShortName=SF                                                     |
//| Class=CSignalSessionFilter                                       |
//| Page=sessionfilter                                               |
//| Parameter=TradeLondonSession,bool,false,Trade London Session     |
//| Parameter=TradeNewYorkSession,bool,true,Trade New York Session   |
//| Parameter=TradeTokyoSession,bool,false,Trade Tokyo Session       |
//+------------------------------------------------------------------+
// wizard description end

//+------------------------------------------------------------------+
//| Class CSignalSessionFilter                                        |
//| Appointment: Class trading signals session filter.                |
//|              Derives from class CExpertSignal.                   |
//+------------------------------------------------------------------+
class CSignalSessionFilter : public CExpertSignalCustom
  {
protected:
   //--- input parameters
   bool              trade_LondonSession;
   bool              trade_NewYorkSession;
   bool              trade_TokyoSession;
//+------------------------------------------------------------------+
//| Check if specified time is within the set time                   |
//+------------------------------------------------------------------+
   bool              inTimeInterval(datetime t, int From_Hour, int From_Min, int To_Hour, int To_Min)
     {
      MqlDateTime gmtTime;
      TimeGMT(gmtTime);
      int hour = gmtTime.hour;
      int minute = gmtTime.min;
      if((hour > From_Hour || (hour == From_Hour && minute >= From_Min)) &&
         (hour < To_Hour || (hour == To_Hour && minute <= To_Min)))
        {
         return true;
        }
      return false;
     }
//+------------------------------------------------------------------+
//| Check if current time is within a specified trading session      |
//+------------------------------------------------------------------+
   bool              inTradingSession(datetime t, string session)
     {
      if(session == "LondonSession")
         return trade_LondonSession && inTimeInterval(t, 8, 0, 16, 0);
      else
         if(session == "NewYorkSession")
            return trade_NewYorkSession && inTimeInterval(t, 13, 0, 22, 0);
         else
            if(session == "TokyoSession")
               return trade_TokyoSession && inTimeInterval(t, 0, 0, 9, 0);
            else
               return false;
     }
public:
                     CSignalSessionFilter(void);
                    ~CSignalSessionFilter(void);

   //--- methods initialize protected data
   void              TradeLondonSession(bool value) { trade_LondonSession = value; }
   void              TradeNewYorkSession(bool value) { trade_NewYorkSession = value; }
   void              TradeTokyoSession(bool value) { trade_TokyoSession = value; }

   //--- method for checking conditions of entering the market
   virtual double    Direction(void);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSignalSessionFilter::CSignalSessionFilter(void)
  {
// Constructor body (if any specific initialization is required)
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSignalSessionFilter::~CSignalSessionFilter(void)
  {
// Destructor body (if any specific cleanup is required)
  }
//+------------------------------------------------------------------+
//| Check if current time is within any specified trading session    |
//+------------------------------------------------------------------+
double CSignalSessionFilter::Direction(void)
  {
   datetime gmtct = TimeGMT();
   bool inLondonSession = inTradingSession(gmtct, "LondonSession");
   bool inNewYorkSession = inTradingSession(gmtct, "NewYorkSession");
   bool inTokyoSession = inTradingSession(gmtct, "TokyoSession");
// Check if the current time is within any selected trading session
   if(trade_LondonSession && inLondonSession)
      return 0.0;
   if(trade_NewYorkSession && inNewYorkSession)
      return 0.0;
   if(trade_TokyoSession && inTokyoSession)
      return 0.0;
// Return empty value if not within any selected trading session
   return EMPTY_VALUE;
  }
//+------------------------------------------------------------------+
